'use strict';

const fs=require('fs');
const net=require('net');
const crypto=require('crypto');
const etools=require('ewt-etools');
const protobuf=require('protobufjs');
const protoRoot=protobuf.loadSync(__dirname+'/Plant3.proto');

// 是否是调试模式
let isDebug=false;
process.execArgv.forEach(r=>{if(r.includes('debug')||r.includes('--inspect-brk=')) isDebug=true;});

// 函数集合
const funcDict={
  /* ip地址处理方法，均支持ipv4及ipv6 */
  // 将ip地址转为字节流
  ip2raw:function (ipStr){
    // ipv6方式识别
    if(ipStr.includes(':')){
      const ipRaw=Buffer.allocUnsafe(16);
      const sp=ipStr.split(':');
      const ignLen=8-sp.length;
      let p=0;// 填充下标
      // 遍历分离数组
      for(let i=0;i<sp.length;i++){
        switch (sp[i].length){

        }
      }
    }
    // ipv4方式识别
    else {
      const numArry=ipStr.split('.').map(i=>{return Number.parseInt(i)});
      return Buffer.from(numArry);
    }
  },
  // 将字节流转回ip地址，当转回ipv6时，总是完整地址，非简写
  raw2ip:function (ipRaw){
    // ipv6方法
    if(ipRaw.length>4){
      const arry=new Array(8);
      for(let i=0;i<8;i++){
        const bufcut=ipRaw.splice(i*2,2);
        arry[i]=bufcut.toString('hex');
      }
      return arry.join(':');
    }
    // ipv4方法
    else {
      const arry=new Array(4);
      for(let i=0;i<4;i++) arry[i]=ipRaw[i].toString();
      return arry.join('.');
    }
  },
  /**
   * IP地址匹配度计算，按照二进制位匹配长度计算
   *
   * ipv4完全相等时，值为32
   * ipv6完全相等时，值为128
   * ipv4与ipv6匹配时，总反馈0
   *
   * @param ipA 传入的ip地址a，支持字符串或ipRaw
   * @param ipB 传入的ip地址b，支持字符串或ipRaw
   */
  ipMatch:function (ipA,ipB){
    // 若是字符串，先转流
    if(typeof ipA==='string') ipA=funcDict.ip2raw(ipA);
    if(typeof ipB==='string') ipB=funcDict.ip2raw(ipB);

    // 长度不一致，反馈0
    if(ipA.length!==ipB.length) return 0;

    // 循环判断至相等的位数
    let i=0,j=0;// 相等字节数，异或运算结果
    while(ipA[i]) {
      // 异或计算不为0，则退出循环
      if(j=ipa[i]^ipB[i]) break;
      // 异或结果为0，表明该字节完全相等
      else i++;
    }
    // 将字节数转为位数
    i*=8;
    // 如果有j，则进行补充判定
    if(j){
      i+=8;
      while(j){i--;j=j>>1}
    }
    // 反馈最终的i
    return i;
  },
  // ip地址是否相同
  ipEqual:function (ipA,ipB){
    // 若是字符串，先转流
    if(typeof ipA==='string') ipA=funcDict.ip2raw(ipA);
    if(typeof ipB==='string') ipB=funcDict.ip2raw(ipB);

    // 流比较
    return ipA.equals(ipB);
  }
};

// id合法性检验函数
function isIdError(id){
  if(typeof id!=='string') return true; // 必须是字符串
  if(id==='master') return true; // 不能是master
  if(id.length>40||id.length===0) return true; // 长度1~40
}

/* ====== 明文bufMsg消息处理 ====== */
// 构建bufMsg
function bufMsgBuild(buffer,typeCode){
  /* 输入校验 */
  // buffer类别
  if(!Buffer.isBuffer(buffer)) throw new Error('buffer is not Buffer');
  if(buffer.length>67108863) throw new Error('buffer too long, more then 64MB');
  // code必须是0～64之间
  if(!Number.isSafeInteger(typeCode)||typeCode<0||typeCode>63) throw new Error('code Error');
  /* 构建消息头 */
  const head=Buffer.allocUnsafe(8);

  // 0~3  低26位长度，高6位typeCode
  const b0_4=buffer.length+(typeCode<<26);
  head.writeUInt32LE(b0_4,0);
  // 4    随机位
  head[4]=Number.parseInt(Math.random()*256);
  // 5~7 异或校验码
  const checkNum=getCheckNum(buffer,b0_4+head[4]);
  head[5]=checkNum%256;
  head[6]=(checkNum>>>8)%256;
  head[7]=checkNum>>>16;

  // 合并head和buffer，即为输出的bufMsg
  return Buffer.concat([head,buffer]);
}
// 异或校验码计算，3字节
function getCheckNum(buffer,seed) {
  seed=seed%16000000;// 不超过2^24即可
  // 遍历buffer计算
  for(let i=0;i<buffer.length;i+=3){
    // 原始数字
    let num=((buffer[i]||0)<<16)+((buffer[i+1]||0)<<8)+(buffer[i+2]||0);
    // 循环位数
    const move=i%8;
    const high=num>>>move;
    const low=num-(high<<move);
    num=high+(low<<(24-move));

    seed=seed^num;
  }
  // 返回
  return seed;
}

// 二进制流片段缓存，应对分包粘包问题
/*
* 实例化时，需跟int参数限定最大允许消息体长度
*
* 接收：调用方法bufPieceIn传入
*
* 输出：
*   重新定义bufMsgFunc(bufMsg)方法，接收整理好的标准bufMsg
*   重新定义errFunc(errcode)方法，errcode可能列表如下：
*     overMaxLen    - 超出允许最大长度
*
* 触发errFunc或bufMsgFunc后，均会实例均会复位，可继续使用
* */
class c_fluentCache {
  constructor(maxLen){
    // 默认允许最大64MB
    maxLen=maxLen||67108863;
    // 如果最大长度设定有误，报错
    if(!Number.isSafeInteger(maxLen)||maxLen<1||maxLen>67108863) throw new Error('maxLen Error');
    // 最大允许长度
    this.maxLen=maxLen;
    // 头4位缓存，始终复用
    this.head=Buffer.allocUnsafe(4);
    // 对外接口函数，需在外部替换
    this.errFunc=(errcode)=>{}; // 错误触发函数
    this.bufMsgFunc=(bufMsg)=>{}; // 切开的消息体

    // 初次调用复位方法
    this.reSet();
  }
  // 复位
  reSet(){
    // 当前接收指针，p<4时，长度位未完整；p>4时，接收到body
    this.p=0;
    // 消息体积缓存，长度位接收完毕时创建
    this.msgBuf=null
  }
  // 二进制流片段进入
  bufPieceIn(bufPiece){
    // 验证输入，不是Buffer对象则报错
    if(!Buffer.isBuffer(bufPiece)) throw new Error('bufPiece is not Buffer');

    /* ========== 将二进制片段分析赋值到类内部缓存 ========= */
    for(let i=0;i<bufPiece.length;i++) {
      // 当长度位读取完毕时，构建缓存
      if(this.p===4) {
        // 消息体长度，前4字节的低26位为长度
        const bodyLen=this.head.readUInt32LE() & 67108863; // 相当于二进制的26个1
        // 超过允许最大长度，抛出错误
        if(bodyLen>this.maxLen){
          this.reSet(); // 复位
          this.errFunc('overMaxLen'); // 抛出错误
          break; // 退出循环
        }
        // 未超过，准备接收消息体
        else {
          // 构建消息缓存
          this.msgBuf=Buffer.allocUnsafe(8+bodyLen); // 前置码一共8位，前4位是长度位
          // 将长度位写入
          for(let i=0;i<4;i++) this.msgBuf[i]=this.head[i];
        }
      }


      // p<4时，读取长度位
      if(this.p<4) {
        this.head[this.p]=bufPiece[i];
        this.p++; // 指针位后移
      }
      // p>=4时，读取消息体
      else {
        this.msgBuf[this.p]=bufPiece[i];
        this.p++; // 指针位后移

        // 如果已经接收到最后，触发消息，并复位
        if(this.p===this.msgBuf.length) {
          this.bufMsgFunc(this.msgBuf); // 触发
          this.reSet(); // 复位
        }
      }
    }
  }
}

/* ====== 公用基类 ====== */

// protoWork类及实例
class c_protoWork{
  constructor(root){
    this._root=root;
  }

  /**
   * 将json序列化为指定格式
   * @param json 必须是字典
   * @param typ 指定的格式字符串
   */
  encode(json,typ){
    // json必须是字典
    if(etools.etype(json)!=='dict') throw new Error('json is not Dict');
    // 预备proto格式
    const protoObj=this._root.lookupType(typ);
    // 确认格式
    const errMsg=protoObj.verify(json);
    if(errMsg) throw new Error('json format Error with type: '+typ+'  Json: '+JSON.stringify(json));
    // 实例化消息
    const mid=protoObj.create(json);
    // 取得流并反馈
    return protoObj.encode(mid).finish();
  }

  decode(buf,typ){
    // 预备proto格式
    const protoObj=this._root.lookupType(typ);
    // 解析并反馈
    return protoObj.decode(buf);
  }

  valueEncode(obj,pArry){
    // 防嵌套数组，递归时传递，没有则新建
    if(!pArry) pArry=[];

    // 获取原型类别
    const objTyp=etools.protoType(obj);

    // 根据etype递归调用
    switch (objTyp) {
      case '[object Null]':return {typ:'0'};
      case '[object Undefined]':return {typ:'1'};
      case '[object Number]':
        if(Number.isSafeInteger(obj)) return {typ:'2',raw:this.encode({obj},'ProtoBuf.Plant3.v_int')};
        else return {typ:'3',raw:this.encode({obj},'ProtoBuf.Plant3.v_float')};
      case '[object Array]':
        const array=new Array(obj.length);
        for(let i=0;i<obj.length;i++)
          array[i]=this.valueEncode(obj[i],pArry);
        return {typ:'4',raw:this.encode({obj:array},'ProtoBuf.Plant3.v_array')};
      // buffer
      case '[object Uint8Array]':
        return {typ:'5',raw:this.encode({obj},'ProtoBuf.Plant3.v_bytes')};
      case '[object String]':
        return {typ:'6',raw:this.encode({obj},'ProtoBuf.Plant3.v_string')};
      case '[object Object]':
        // 类别优先使用指定的，若没有则使用原型名
        const protoTyp=obj._protoBufType||obj.constructor.name;
        // 普通对象
        if(protoTyp==='Object'){
          // 嵌套检查，有嵌套则报错
          if(pArry.includes(obj)) throw new Error('obj 嵌套错误');
          pArry.push(obj);

          // 递归调用
          const dictArry=[];
          for(const key in obj){
            // 忽略 __proto__项目
            if(key==='__proto__') continue;
            dictArry.push({key,val:this.valueEncode(obj[key],pArry)});
          }
          return {typ:'7',raw:this.encode({obj:dictArry},'ProtoBuf.Plant3.v_dict')};
        }
        // ProtoBuf对象
        else {
          // 不能删除 delete obj._protoBufType;
          return {typ:protoTyp,raw:this.encode(obj,protoTyp)};
        }
      case '[object Boolean]':
        // true false 分别占用一个单独的类型代码
        return {typ:obj?'8':'9'};
      default:
        throw new Error('unsport format: '+protoTyp);
    }
  }

  valueDecode(paramItem){
    switch (paramItem.typ){
      case '0':return null;
      case '1':return;// undefined
      case '8':return true;
      case '9':return false;
      case '2':
        const obj=this.decode(paramItem.raw,'ProtoBuf.Plant3.v_int').obj;
        // 后台需要toNumber方法，前端不能执行
        return (typeof obj==='number')?obj:obj.toNumber();
      case '3':
        return this.decode(paramItem.raw,'ProtoBuf.Plant3.v_float').obj;
      case '4':
        // 先提取成数组
        const arry=this.decode(paramItem.raw,'ProtoBuf.Plant3.v_array').obj;
        // 再迭代还原
        return arry.map(r=>{return this.valueDecode(r)});
      case '5':
        return this.decode(paramItem.raw,'ProtoBuf.Plant3.v_bytes').obj;
      case '6':
        return this.decode(paramItem.raw,'ProtoBuf.Plant3.v_string').obj;
      case '7':
        // 先提取成数组
        const dictArry=this.decode(paramItem.raw,'ProtoBuf.Plant3.v_dict').obj;
        const result={};
        dictArry.forEach(r=>result[r.key]=this.valueDecode(r.val));
        return result;
      default:
        return this.decode(paramItem.raw,paramItem.typ);
    }
  }
}
const protoWork=new c_protoWork(protoRoot);

/**
 * api机制通讯类
 *
 * socket可以是后台的socket或者前端的ws
 * 需提前提供的方法：
 * sendProtoBuf - 发送二进制流，外层可继续封装
 * destroy - 销毁方法，销毁后内部的isSocketOk方法一定要能判定出销毁了
 */
class c_apiConn {
  constructor(socket,apiFunc,showName) {
    this.socket=socket;
    // 唯一函数处理方法，
    if(typeof apiFunc==='function') this.staticFunc=apiFunc;
    // 函数搜索类方法
    else this.apiFunc=apiFunc;
    this.showName=showName || 'apiConn';
    this.protoWork=protoWork;
    // 最后收到消息的时间戳，用于心跳，默认用当前时间
    this._lastRecSt=Date.now();

    // 调试开关，默认为关闭
    this.debug=false;
    // 对外调用count计数器
    this._count = 1;
    // 己方对外请求的缓存
    this._reqCache = {
      // key 发出时己方生成的count
      // value 数组三项：[sec, apiReqBuf, callback]
    };
    // 对方请求，己方运行完毕的结果缓存
    this._resCache = {
      // key 对方请求的count
      // value 数组三项：[sec, apiResBuf]
      // 其中，json项为null，则表明己方接收到请求但尚未处理完毕；为object时，表明已经处理完毕
    };
    // 内部秒循环
    this._secCircle =setInterval(()=>this._secCircleFunc(),1000);
    // 是否已被销毁
    this._destroyed=false;
  }

  // 秒循环工作函数
  _secCircleFunc() {
    /* 缓存定时清理 */
    // 响应结果保存30秒，超过时间删除
    for (const count in this._resCache) {
      // 获取发送对象
      const resObj = this._resCache[count];
      // 计时器累加
      resObj[0]++;
      // 超时删除
      if (resObj[0] >= 30) {
        // 调试输出
        if (this.debug) etools.log(`[${this.showName}] [DEBUG] API-RES 缓存到期，删除，count: ` + count);

        delete this._resCache[count];
      }
    }

    // 请求收到反馈时已经删除，未收到时在缓存里
    for (const count in this._reqCache) {
      // 获取发送对象
      const reqObj = this._reqCache[count];
      // 计时器累加
      reqObj[0]++;
      // 第5、15秒重新发送
      if (this.isSocketOk() && (reqObj[0] === 5 || reqObj[0] === 15)) {
        this.socket.sendProtoBuf(reqObj[1]);

        // 调试输出
        if (this.debug) etools.log(`[${this.showName}] [DEBUG] API-REQ 重发请求，count: ` + count);
      }
      // 超过30秒删除，执行回调函数
      if (reqObj[0] >= 30) {
        delete this._reqCache[count]; // 从字典中删除
        reqObj[2] && reqObj[2]('timeout'); // 执行回调函数，报超时错误

        // 调试输出
        if (this.debug) etools.log(`[${this.showName}] [DEBUG] API-REQ 请求超时，count: ` + count);
      }
    }

    /* 心跳发送与检验 */
    const now = Date.now();
    // socket存在且最后收到消息时间超过30秒后，每秒都发送ping
    if (this.isSocketOk() && now - this._lastRecSt > 30e3) {
      // 组织请求体
      const apiReqJson = {
        count: this._getCount(),
        cmd: 'ping'
      };
      // 序列化
      const apiReqBuf = this.protoWork.encode(apiReqJson, 'ProtoBuf.Plant3.bufMsg_3_api');
      // 发送
      this.socket.sendProtoBuf(apiReqBuf);
    }
    // 超过40秒，销毁socket并设定为null
    if (this.socket && now - this._lastRecSt > 40e3){
      // socket和ws的关闭方法都尝试执行
      if(typeof this.socket.destroy==='function') this.socket.destroy();
      if(typeof this.socket.close  ==='function') this.socket.close();

      // 清除记录
      this.socket=null;
    }
  }

  // 判定socket是否可用
  isSocketOk() {
    // 获取通讯对象，可能是socket或ws
    const obj=this.socket;
    // 无对象
    if(!obj) return false;

    // socket 连接中
    if(obj.connecting===true) return false;

    // socket不可写或不可读
    if(obj.readable===false||obj.writable===false) return false;

    // 各种检测通过，反馈true
    return true;
  }

  // 获取请求序号
  _getCount(){
    // 获取序号
    const count = this._count;
    if (count < 214e7) this._count++; // 计数器累加
    else this._count = 1; // 越界归1

    return count;
  }

  // 接到对方调用API请求
  _rec_apiReq(apiReq){
    // 如果已经销毁，不再接收消息
    if(this._destroyed) return;

    // 如果有cmd，则特殊处理
    if(apiReq.cmd){
      // 构造基础反馈体
      const apiRes = {
        count: -apiReq.count, // 负数表示反馈
        cmd:''                // 反馈cmd，暂为空字符串
      };

      // 识别命令
      switch (apiReq.cmd){
        // 收到心跳ping，回pong
        case 'ping':
          apiRes.cmd='pong';
          break
      }

      // 组织反馈
      if(this.isSocketOk()){
        const apiResBuf=this.protoWork.encode(apiRes,'ProtoBuf.Plant3.bufMsg_3_api');
        this.socket.sendProtoBuf(apiResBuf);
      }

      return;
    }

    // 普通 API调用请求

    /* 根据count判断该此请求要怎么处理 */
    const resCache = this._resCache[apiReq.count];

    // 新请求，缓存字典中无记录，着手处理该请求
    if(!resCache){
      // 调试输出
      if (this.debug) etools.log(`[${this.showName}] [DEBUG] API-RES 新请求(${apiReq.api})，建立缓存，执行，count: `+apiReq.count);
      // 首先在resCache字典中，建立记录
      const resCache = this._resCache[apiReq.count]=[0,null,null];

      // 获取处理函数
      const func=this.staticFunc||this.apiFunc.getFunc(apiReq.api);

      // 构造info对象
      const info={
        socket:this.socket,
        api:apiReq.api,
        heads:{}
      };
      // 写入heads信息
      for(let i=0;i<apiReq.heads.length;i+=2) info.heads[apiReq.heads[i]]=apiReq.heads[i+1];

      // 执行
      func(info,apiReq.params.map(b=>this.protoWork.valueDecode(b)),(...all)=>{
        // 构造反馈体
        const apiRes = {
          count: -apiReq.count, // 负数表示反馈
          params:all.map(b=>this.protoWork.valueEncode(b))
        };

        // 将apiRes整体序列化
        const apiResBuf=this.protoWork.encode(apiRes,'ProtoBuf.Plant3.bufMsg_3_api');

        // 将反馈结果存入缓存
        resCache[1]=apiResBuf;

        // 判断连接状态
        if(this.isSocketOk()){
          // 将处理结果发送出去
          this.socket.sendProtoBuf(apiResBuf);
          // 调试输出
          if (this.debug) etools.log(`[${this.showName}] [DEBUG] API-RES 处理完毕，填充缓存，初次发送，count: `+apiReq.count);
        }
        // 无法发送
        else if (this.debug) {
          etools.log(`[${this.showName}] [DEBUG] API-RES 处理完毕，填充缓存，初次发送失败，count: `+apiReq.count);
        }
      });
    }
    // 已经接受到请求，且已经处理完毕，再次发送处理结果
    else if (resCache[1]){
      // 判断连接状态
      if(this.isSocketOk()){
        // 再次将处理结果发送出去
        this.socket.sendProtoBuf(resCache[1]);
        // 调试输出
        if (this.debug) etools.log('[Plant] [DEBUG] API-RES 重复请求，已处理完毕，再次发送，count: '+apiReq.count);
      }
      // 无法发送
      else if(this.debug){
        etools.log('[Plant] [DEBUG] API-RES 重复请求，已处理完毕，再次发送失败，count: '+apiReq.count);
      }
    }
    // 已经接受到请求，但尚未处理完毕，不用任何处理
    else {
      // 调试输出
      if (this.debug) etools.log('[Plant] [DEBUG] API-RES 重复请求，尚未处理完毕，不响应，count: '+apiReq.count);
    }
  }

  // 接到己方请求的反馈
  _rec_apiRes(apiRes){
    // 如果已经销毁，不再接收消息
    if(this._destroyed) return false;

    // 如果有cmd，则特殊处理
    if(apiRes.cmd){
      // 识别命令
      switch (apiRes.cmd){
        // 收到心跳pong，不做任何处理
        case 'pong':break
      }

      return;
    }

    // 从缓存取出请求对象
    const reqObj=this._reqCache[apiRes.count];
    // 取出成功
    if(reqObj){
      // 从缓存中删除该记录
      delete this._reqCache[apiRes.count];

      // 有回调函数，解析执行
      if(reqObj[2]){
        reqObj[2](...apiRes.params.map(b=>this.protoWork.valueDecode(b)));
      }

      // 调试输出
      if (this.debug) etools.log('[Plant] [DEBUG] API-REQ 请求响应成功，count: '+apiRes.count);
    }
    // 取出失败意味着已经超时，不用管了
  }

  // 接收protoBuf流
  recProtoBuf(buf){
    // 记录收到消息时间戳
    this._lastRecSt=Date.now();
    // 解析
    const apiObj = this.protoWork.decode(buf,'ProtoBuf.Plant3.bufMsg_3_api');
    // 请求处理
    if(apiObj.count>0) this._rec_apiReq(apiObj);
    // 反馈处理
    else{
      apiObj.count=-apiObj.count;
      this._rec_apiRes(apiObj);
    }
  }

  /**
   * 调用对方API请求
   *
   * @param apiObj string-API名；object-详细字典
   * @param params 参数数组，若为单个，则内部包装成数组
   * @param callback 回调函数，可以为空
   */
  callApi(apiObj,params,callback) {
    /* api请求信息解析 */
    let api,heads;
    // 字符串格式的支持
    if(typeof apiObj==='string'){
      heads={};
      api=apiObj;
    }
    // 对象形式的支持
    else {
      api=apiObj.api;
      heads=apiObj.heads||{};
    }

    // 请求参数处理，必须是数组
    if(!Array.isArray(params)) params=[params];

    // 包装请求体
    const reqJson={
      count:this._getCount(),
      api,
      heads:[],
      params:new Array(params.length)
    };

    // 写入请求头信息
    for(const key in heads){
      if(!heads.hasOwnProperty(key)) continue;// 跳过非属性项
      const value=heads[key];
      const typ=typeof value;
      let str;
      // 可传输的类型
      if(typ==='number'||typ==='boolean') str=value.toString();
      else if(typ==='string') str=value;
      else if(value===null) str='null';
      // 不可传输，跳过
      else continue;

      // 写入
      reqJson.heads.push(key,str)
    }

    // 写入参数
    for(let i=0;i<params.length;i++)
      reqJson.params[i]=protoWork.valueEncode(params[i]);

    // 生成流
    const apiReqBuf=this.protoWork.encode(reqJson,'ProtoBuf.Plant3.bufMsg_3_api');

    // 缓存请求
    this._reqCache[reqJson.count] = [0, apiReqBuf,callback];

    // 初次发送请求
    if(this.isSocketOk()){
      this.socket.sendProtoBuf(apiReqBuf);// 调试日志
      if(this.debug) etools.log('[Plant] [DEBUG] API-REQ 发出新请求，count: '+reqJson.count);
    }
    else if(this.debug){
      if(this.debug) etools.log('[Plant] [DEBUG] API-REQ 发出新请求失败，count: '+reqJson.count);
    }
  }

  // 销毁
  destroy() {
    // 标记已经销毁
    this._destroyed=true;
    // 清空定时器
    clearInterval(this._secCircle);
  }
}
// api机制处理程序注册查询类
class c_apiFunc {
  constructor(){
    // 对外提供的API字典集合（高优先级）
    this._apiDict = {};
    // 对外提供的API正则匹配数组（低优先级，有顺序）
    this._apiRegex = [];
  }
  // 设定对外提供的API
  reg(api, func) {
    // func必须是函数，否则为编程错误
    if(typeof func!=='function') throw new Error('func format Error');

    // 根据api类型，来确定设定方法
    switch (etools.etype(api)){
      // 字符串：记录到字典匹配
      case 'string':
        this._apiDict[api] = func;
        break;
      // 正则对象，添加到正则匹配数组
      case 'regex':
        this._apiRegex.push([api,func]);
        break;
      // 未识别
      default: throw new Error('unknow how to regApi');
    }
  }

  // 获取方法函数
  getFunc(api){
    // api必须是字符串，否则为编程错误
    if(typeof api!=='string') throw new Error('api must be String');

    // 优先级 1：如果有字典匹配，直接反馈
    if(api in this._apiDict) return this._apiDict[api];

    // 优先级 2：依次尝试正则匹配，匹配成功，则反馈
    for(let i=0;i<this._apiRegex.length;i++)
      if(this._apiRegex[i][0].test(api)) return this._apiRegex[i][1];

    // 末次：返回未找到方法
    const func=(info,[], callback)=>{
      // 作为API方法集用时，callback位置为function，需报错
      if(typeof callback==='function') callback('noApi: ' + api)
    };
    return func;
  }
}
// A005304444412
/**
 * 事件机制类，依赖于netPoint
 *
 * 需用到netPoint中的以下方法、对象：
 *  apiFunc实例，用于注册特定方法函数
 *  callApi()方法，用于向订阅者推送值
 *
 */
class c_event{
  constructor(netPoint){
    this.debug=true;
    // 事件值获取函数
    this._netPoint=netPoint;

    // 对方订阅请求
    netPoint.apiFunc.reg('_eventBook',(...all)=>this._api_eventBook(...all));
    // 对方触发事件
    netPoint.apiFunc.reg('_eventEmit',(...all)=>this._api_eventEmit(...all));

    // 己方向外订阅字典
    /*
    * key为全事件名，含对方服务名
    * 值字典中：
    *   err
    *   state     0 订阅失败或未订阅，1 订阅中，2 订阅API调用正常
    *   serviceId
    *   eventName
    *   funcs      - 数组，订阅函数集合
    *   paramsArry - 订阅结果数组，既最近emit数组，若为null则表明之前没有emit过
    * */
    this._books={};

    // 外方向己方订阅的事件名字典
    /*
     * key为己方事件名
     * value字典中：
     *    paramsArry：值数组
     *    bookers：订阅服务id数组
     */
    this._emits={};

    /**
     * 订阅验证函数
     *
     * 默认在err位置返回false,，即允许任何订阅
     * @param eventName 事件名
     * @param id 订阅者标识
     * @param callback
     */
    this.bookCheck=(eventName,id,callback)=>{
      callback();
    };
    /**
     * 彻底取消订阅，即该事件没有任何id在订阅
     */
    this.cancel=(eventName)=>{};
  }

  // 订阅请求处理
  _api_eventBook(info,[eventName],callback){
    const id=info.socket.id;
    // 调用验证函数，判定是否允许订阅
    this.bookCheck(eventName,id,(err)=>{
      // 不允许订阅则结束，允许则callback时err实际为空
      callback(err);
      if(err) return;

      // 调试日志
      this._netPoint.log(`[Event Booked] ${eventName} @ ${id}`);

      /* 记录订阅 */
      // emit字典中没有记录，创建
      let emitObj=this._emits[eventName];
      if(!emitObj) emitObj=this._emits[eventName]={bookers:[]};

      // 有数组，且未包含该id，添加
      if(!emitObj.bookers.includes(id)) emitObj.bookers.push(id);

      // 如果当前推送记录有值，进行一次推送，不考虑在此推送失败的情况
      if(emitObj.paramsArry) info.socket.callApi('_eventEmit',[eventName,...emitObj.paramsArry],()=>{});
    })
  }

  // 对方触发事件
  _api_eventEmit(info,[eventName,...paramsArry],callback){
    const event=info.socket.id+'.'+eventName;
    const bookObj=this._books[event];
    // 如果己方没有订阅，则callback错误
    if(!bookObj) callback('noBook');
    // 己方有订阅
    else {
      // 记录本次推送的结果
      bookObj.paramsArry=paramsArry;
      // 记录最初数组长度
      const len=bookObj.funcs.length;
      // 严格按照最初的数组长度，确定循环次数
      for(let i=0;i<len;i++){
        // 从头部取出一个func
        const func=bookObj.funcs.shift();
        // 执行，若反馈判定为真，再添加回尾部，判定为假，不再加入，即已删除
        if(func(...paramsArry)) bookObj.funcs.push(func);
      }
      // 如果最终数组长度大于0，空反馈，即继续订阅
      if(bookObj.funcs.length>0) callback();
      // 没有func了，反馈err取消订阅
      else {
        // 取消订阅
        callback('noBook');
        // 删除己方对该事件的记录节点
        delete this._books[event];
      }
    }
  }

  /**
   * 重新定于指定service下的所有事件
   * @param serviceId 服务id
   * @param forceReBook 是否强制重新订阅
   * @private
   *
   */
  reBookService (serviceId,forceReBook) {
    // 遍历所有当前订阅的事件
    for(const id in this._books){
      const bookObj=this._books[id];
      // 判定是否重新订阅
      if(bookObj.serviceId===serviceId && // 服务名与此次通知匹配
        bookObj.state!==1 &&              // 不在订阅过程中
        (forceReBook||bookObj.state===0)  // 强制重新订阅，或当前未订阅
      )
        this._bookCall(bookObj);
    }
  }

  /**
   * 向对应服务发送订阅请求，并更新内部标记状态
   * @param bookObj 订阅状态对象
   * @private
   */
  _bookCall(bookObj){
    // 标记订阅中
    bookObj.state=1;

    // 调用api机制向对方订阅
    this._netPoint.callApi(bookObj.serviceId+'._eventBook',bookObj.eventName,(err)=>{
      // 订阅错误标记
      if(err){
        etools.log(`[${this._netPoint.id}] [Event - Book Fail   ]: ${bookObj.serviceId}.${bookObj.eventName}`);
        bookObj.state=0;
        bookObj.err=err;
      }
      // 订阅成功
      else {
        etools.log(`[${this._netPoint.id}] [Event - Book Success]: ${bookObj.serviceId}.${bookObj.eventName}`);
        bookObj.state=2;
        bookObj.err=null;
      }
    })
  }

  // 对外方法：订阅
  book(event,func){
    // 事件名非字符串，程序编写错误
    if((typeof event!=='string')||!event) throw new Error('eventName must be string and not empty');

    // 订阅函数类型
    if(typeof func!=="function") throw new Error('func typeError');

    // 尝试读取已有订阅信息
    let bookObj=this._books[event];

    // 已经订阅，无论订阅状态
    if(bookObj){
      // 缓存func
      bookObj.funcs.push(func);
      // 如果之前曾推送过结果，对本func触发一次
      if(bookObj.paramsArry) func(...bookObj.paramsArry);
    }
    // 新订阅
    else {
      // 分离服务名称和事件名称
      const index=event.indexOf('.');
      if(!index>0) throw new Error('fullEventName: serviceId + eventName');
      const serviceId=event.substr(0,index);
      const eventName=event.substr(index+1);

      // 新建事件本地存储，paramsArry为空
      bookObj=this._books[event]={
        serviceId,eventName,
        state:0,        // 0 未订阅或订阅失败;  1 订阅成功
        verSt:null,     // 最近尝试时的对方版本 null表示未曾订阅，0表示订阅时未知对方时间戳
        err:null,       // 最近一次的错误信息，重新订阅成功时不清除
        funcs:[func],   // 订阅的func数组
        paramsArry:null // 最后一次推送的订阅信息
      };
      // 立即订阅
      this._bookCall(bookObj);
    }
  }

  // 触发某事件的推送，值需要从外部字典获取，若无值，则不必继续
  emit(eventName,...paramsArry){

    /* 记录订阅 */
    // emit字典中没有记录，创建
    let emitObj=this._emits[eventName];
    if(!emitObj) emitObj=this._emits[eventName]={bookers:[]};

    // 记录值，为以后新订阅初次发送准备
    emitObj.paramsArry=paramsArry.length>0?paramsArry:null;

    // 遍历bookers数组进行推送
    emitObj.bookers.forEach(id=>{
      // 调用对方API
      this._netPoint.callApi(id+'._eventEmit',[eventName,...paramsArry],(err)=>{
        // 推送无错误，可结束
        if(!err) return;
        // 推送有错误，判定错误原因

        // 取消订阅
        const index=emitObj.bookers.indexOf(id);
        if(index>=0){
          // 从数组中移除id
          emitObj.bookers.splice(index,1);
          // 调试日志
          if(isDebug) etools.log(`[${this._netPoint.id}] [Event Unbook]: ${eventName} @ ${id}`);
        }
      })
    });
  }
}

// 通讯基础类，每个socket建立一个
class c_econn{
  constructor(socket,apiFunc){
    /* 消息接收 */
    this.socket=socket;
    // 分配分包粘包类
    const cache=new c_fluentCache(); // 默认限制极大允许64兆
    // API机制类
    this.api=new c_apiConn(socket,apiFunc,'Plant');
    // this.api.debug=true;
    socket.on('error',(err)=>{
      etools.log('连接断开')
    });
    socket.on('close',()=>{
      this.api.destroy();
    });
    // TCP层数据接收
    socket.on('data',(buf)=>{
      cache.bufPieceIn(buf);
    });
    // 分包粘包输出bufMsg
    cache.bufMsgFunc=(bufMsg)=>{
      this._recBufMsg(bufMsg);
    };
    // 分包粘包出错，关闭socket
    cache.errFunc=(err)=>{
      socket.destroy();
    };
    // 增加发送bufMsg函数
    socket.sendBufMsg=(buf,typeCode)=>{
      if(typeCode>=24) throw new Error('sendBufMsg方法只能用于发送明文消息');
      // 如果socket已经销毁，不再继续
      if(socket.destroyed || !socket.writable) return;

      // 明文包装
      let bufMsg=bufMsgBuild(buf,typeCode);
      // 如果有证书，进行加密
      if(socket.certBuf){
        const key=Buffer.concat([socket.skeyBuf,socket.certBuf]);
        let cryptBuf;
        try{
          let cip = crypto.createCipher('aes-128-ecb', key);
          cryptBuf = Buffer.concat([cip.update(bufMsg),cip.final()]);
        } catch (e){
          // 遇到错误，销毁链接
          socket.destroy();
          return;
        }
        // 加密包装
        bufMsg=bufMsgBuild(cryptBuf,24);
      }

      // 发送
      this.socket.write(bufMsg);
    };
    // 增加发送protoBuf方法
    this.socket.sendProtoBuf=(buf)=>this.socket.sendBufMsg(buf,3);
    // 增加调用API方法
    this.socket.callApi=(...all)=>this.api.callApi(...all);
  }
  // 1、接收到bufMsg
  _recBufMsg(bufMsg){
    // 校验码比对
    const b0_4=bufMsg.readUInt32LE(0);
    const buf=bufMsg.slice(8);
    const jsNum=getCheckNum(buf,b0_4+bufMsg[4]);// 计算校验码
    const csNum=bufMsg[5]+(bufMsg[6]<<8)+(bufMsg[7]<<16);// 传输校验码
    // 若比对错误，销毁socket
    if(jsNum!==csNum){
      this.socket.destroy();
      return
    }
    // 分离typeCode，并分类处理
    const typeCode=b0_4>>>26;
    switch(typeCode){
      case 3: // 明文：API机制消息，使用protobuf
        this.api.recProtoBuf(buf);
        break;
      case 24: // 加密包，解密后迭代
        // 如果本地socket没有设定加密证书，直接断开
        if(!this.socket.certBuf){
          etools.log('socket 没有设定证书，但收到了加密消息，连接断开');
          this.socket.destroy();
        }
        // 有证书，可以进行解密
        else {
          const key=Buffer.concat([this.socket.skeyBuf,this.socket.certBuf]);
          const decipher = crypto.createDecipher('aes-128-ecb', key);
          let newbuf;
          try{
            const buf_update=decipher.update(buf);
            const buf_final =decipher.final();
            newbuf=Buffer.concat([buf_update,buf_final]);
          } catch (e) {
            etools.log('socket 解密过程出错，连接断开');
            this.socket.destroy();
            return;
          }
          // 解密完成，迭代
          this._recBufMsg(newbuf);
        }
        break;
      case 25: // gzip压缩包
        debugger;
        break;
      default:
        socket.emit('unknowBufMsg',bufMsg);
        break;
    }
  }
}

class c_netPoint{
  constructor(){
    /**
     * 其他服务连接字典
     *
     * key为各服务id，value字典为：
     *  socket     : 连接实例，有此项则连接应该可用
     *  verSt      : 当前或之前在线的socket的版本
     *  empSec     : 通讯空闲时间，用于触发心跳机制
     *  offCount   : 连续失败断线次数，一旦有成功登录，则归零
     *
     *  【service中项目】
     *  connKey：连接确认秘钥
     *  calkArry：等待缓存数组，有则意味着在连接中
     *  tryAgainSt : 允许再次尝试连接的时间戳
     *
     *  【master中项目】
     *  info：连接信息
     *  mate_[id]：与同伴服务建立连接的信息，彼此同引用
     *    st：尝试时间戳
     */
    this._connDict={};
    // tcp监听实例
    this._tcpServer=null;
    // api方法集合，及内部方法订阅
    this.apiFunc=new c_apiFunc();
    this.apiFunc.reg(/^_{2}/,(info,params,callback)=>{
      const func=this['_api'+info.api];
      // 如果匹配到函数，执行
      if(typeof func==='function') func.apply(this,[info,params,callback]);
      // 未匹配到，反馈错误
      else callback('unknow inner Api');
    });
    // event事件类
    this.event=new c_event(this);
    // 版本时间戳，仅起到版本的作用
    this.verSt=(Date.now() & 2147483647)+1;
  }

  // 获取监听端口
  get _tcpPort(){
    if(!this._tcpServer) return 0;
    else return this._tcpServer.address().port;
  }

  // 构建监听端口
  _tcpListenBuild(){
    let tryPort;

    // 获取本次尝试的端口
    const getPort=()=>{
      // 设定为数组
      if(Array.isArray(this._listenPort)){
        // 第一次请求时，取范围下限
        if(!tryPort) tryPort=this._listenPort[0];

        // 如果tryPort未越界，反馈其值并++运算，越界则自然return空
        if(tryPort<=this._listenPort[1]) return tryPort++;
      }
      // 设定为确切数字
      else if(this._listenPort){
        // 如果之前没尝试过，反馈设定值，尝试过，则自然return空
        if(!tryPort) {
          tryPort=this._listenPort;
          return tryPort;
        }
      }
      // 没有设定，使用3w以上的随机值
      else return Math.floor(Math.random()*35536)+30000;
    };

    // 监听成功后的方法
    const listenOk=(server)=>{
      // 登记
      this._tcpServer=server;

      // 日志输出
      this.log(`TCP Listing at: `+this._tcpPort);

      // 新连接到达，调用socket初始化方法
      server.on('connection',(socket)=>this._socketInit(socket));

      // 如果不是master，则调用连接master方法
      if(this.id!=='master') this._connMaster();
    };

    // 尝试建立监听方法
    const tryListen=()=>{
      const port=getPort();
      if(!port) throw new Error('build Listen Error');

      const server=net.createServer();    // 创建实例
      server.listen(port);    // 开启监听

      // 是否已判定失败
      let failed=false;

      // 监听关闭、错误事件，均触发再次尝试
      server.on('close',()=>{
        if(!failed){
          failed=true;
          tryListen();
        }
      });
      server.on('error',()=>{
        if(!failed){
          failed=true;
          tryListen();
        }
      });
      // 监听成功，则调用listenOk方法
      server.on('listening',()=>listenOk(server));
    };

    // 初次尝试建立
    tryListen();
  }

  // 原始socket对象丰富功能，但未标记认证信息
  _socketInit(socket){
    socket.econn=new c_econn(socket,this.apiFunc);
  }

  /**
   * 可信socket上线
   * @param socket 连接实例
   * @param verSt 该链接版本
   * @param skey 会话秘钥
   * @param cert 加密证书，非加密时为空
   * @private
   */
  _socket_on(socket,verSt,skey,cert){
    // 若加密，则登记信息
    if(cert){
      socket.skeyBuf=skey;
      socket.certBuf=cert;
    }

    // 取出id方便使用
    const id=socket.id;

    // 写日志
    this.log(`socket on : `+id);

    // 获取可信字典对象，没有则创建
    let econn=this._connDict[id];
    if(!econn) econn=this._connDict[id]={};

    // 空闲时间，失败连接次数，允许重连时间戳归0
    econn.empSec=0;
    econn.offCount=0;

    // 登记socket
    econn.socket=socket;

    // 调用服务启动方法
    this._serviceOn(socket.id,verSt,verSt!==econn.verSt);

    // 登记此版本
    econn.verSt=verSt;

    // 取回调集合并清空
    const arry=econn.calkArry;
    econn.calkArry=null;
    // 若有等待回调，则执行
    if(arry) arry.forEach(f=>f(undefined,socket));

    // 有任何数据到达，空闲时间归零
    socket.on('data',()=>{
      if(econn.socket===socket)
        econn.empSec=0
    });

    // 断线取消登记
    socket.on('close',()=>{
      this.log(`socket off: ${id} verSt: ${verSt}`);
      // 如果此版本即当前版本，清除socket
      if(econn.socket===socket) econn.socket=null;
      // 销毁socket
      socket.destroy();
    });

    // 重新订阅相关事件
    this.event.reBookService(id,true);
  }

  // 调用API方法
  callApi(apiObj,params,callback){
    // 灵活调用格式
    if(typeof params==="function"){
      callback=params;
      params=[];
    }
    if(!callback) callback=()=>{};

    // 字符串封装为标准对象
    if(typeof apiObj==='string') apiObj={api:apiObj};
    // 分离服务名
    const index=apiObj.api.indexOf('.');
    const sname=apiObj.api.substr(0,index);
    apiObj.api=apiObj.api.substr(index+1);
    // 获取eConn对象
    this._getSocket(sname,(err,socket)=>{
      if(err) callback('get eConn Error: '+err);
      else socket.callApi(apiObj,params,callback);
    })
  }

  // 判定指定id是否在线
  isIdOn(id){
    const econn=this._connDict[id];
    if(econn&&econn.socket) return true;
  }

  // 写一般日志
  log(str){
    etools.log(`[${this.id}] `+str);
  }

  // 写调试日志
  logDebug(str){
    if(isDebug) etools.log(`[${this.id} Debug] `+str);
  }

  // 当前连接的socket字典
  socketDict(){
    const resultDict={};
    for(const id in this._connDict){
      const econn=this._connDict[id];
      const socket=econn && econn.socket;
      if(socket) resultDict[id]=socket;
    }
    return resultDict;
  }
}
class c_master extends c_netPoint{
  constructor(conf){
    super();
    this.id='master';
    this._listenPort=conf.port; // 服务端口，默认随机

    // 是否禁止服务间通讯
    this._forbidMateConn=conf.forbidMateConn;

    // 默认验证函数不加密
    this.regFunc=(post,callback)=>callback(undefined,{});
    // 建立tcp监听，conf中可配置port和host项目
    this._tcpListenBuild();
  }

  /**
   * 服务上线业务方法
   * @param id     服务id
   * @param verSt  版本戳
   * @param isBoot 是否是初次启动
   * @private
   */
  _serviceOn(id,verSt,isBoot){
    // 如果是初次启动，且没有禁止服务间通讯，广播服务可用
    if(isBoot && !this._forbidMateConn){
      this.broadMsg('__serviceUsable',id,verSt);
    }
  }

  // master获取econn实例
  _getSocket(id,callback){
    // 唯一来源是可信dict字典
    const econn=this._connDict[id];
    if(econn&&econn.socket) callback(undefined,econn.socket);
    else callback('no eConn id: '+id);
  }

  // 服务前来注册
  _api__connReg(info,[regObj],callback){
    // 不允许服务名为master
    if(regObj.id==='master'){
      callback('service cannot use id: master');
      return
    }
    // 调用认证函数
    this.regFunc(regObj,(err,json,cert)=>{
      // 无论是否有用，先生成一个会话秘钥
      const skey=crypto.randomBytes(32);

      // 有错误，认证失败
      if(err) {
        callback('reg Fail');
        return;
      }
      // 无证书，不加密
      else if(!cert) callback(undefined,{
        _protoBufType:'ProtoBuf.Plant3.regToMaster_res',
        verSt:this.verSt,
        crypto:false
      });
      // 有证书，下发
      else if(json&&json.sendCert) callback(undefined,{
        _protoBufType:'ProtoBuf.Plant3.regToMaster_res',
        verSt:this.verSt,
        crypto:true,
        cert,skey
      });
      // 有证书，不下发
      else callback(undefined,{
        _protoBufType:'ProtoBuf.Plant3.regToMaster_res',
        verSt:this.verSt,
        crypto:true,
        skey
      });

      // 有效登记
      info.socket.id=regObj.id;
      // 触发上线方法 (非加密时cert为空)
      this._socket_on(info.socket,regObj.verSt,skey,cert);
      // 记录连接信息
      this._connDict[regObj.id].info=regObj;
    })
  }

  // 服务A想要连接服务B
  _api__connMate(info,[post],callback){
    this.logDebug(`${info.socket.id} 想要连接 ${post.mateId}`);

    // 如果禁止服务间通讯，此API反馈错误
    if(this._forbidMateConn) {
      callback('master forbidMateConn');
      return;
    }

    // 获取conn字典中，双方的信息
    const conn_a=this._connDict[info.socket.id];
    const conn_b=this._connDict[post.mateId];

    // 如果b不在线，则callback
    if(!conn_b||!conn_b.socket) {
      callback('service off line: '+post.mateId);
      return;
    }

    // 获取当前时间
    const now=Date.now();

    // 获取建立状态信息
    let buildInfo=conn_a['mate_'+conn_b];

    // 如果在3s内尝试过建立，则此次不再发起
    if(buildInfo&&now-buildInfo.st<3e3) {
      callback();
      return
    }

    // 生成建立状态
    buildInfo=conn_a['mate_'+conn_b]=conn_b['mate_'+conn_a]={
      st:now,
      connKey:post.connKey
    };

    // 先要求b尝试连接a
    conn_b.socket.callApi('__tryToMate',{
      _protoBufType:'ProtoBuf.Plant3.connMate_Build',
      mateId  : conn_a.socket.id,
      connKey : buildInfo.connKey,
      port    : conn_a.info.port,
      ipRaw   : conn_a.info.ipRaw,
      pubAddr : conn_a.info.pubAddr
    },(err)=>{
      // 如果没有错误发生，结束
      if(!err) callback();
      // 有错误发生，callback时带连接对象，要求a连接b
      else callback(undefined,{
        _protoBufType:'ProtoBuf.Plant3.connMate_Build',
        mateId  : conn_b.socket.id,
        connKey : buildInfo.connKey,
        port    : conn_b.info.port,
        ipRaw   : conn_b.info.ipRaw,
        pubAddr : conn_b.info.pubAddr
      })
    })
  }

  // 发起广播消息
  broadMsg(broadObj,...params){
    // 整理
    if(typeof broadObj==='string') broadObj={broad:broadObj};
    // 如果没有broad项目，为编程错误
    if(!broadObj.broad||(typeof broadObj.broad!=='string')) throw new Error('broadMsg Format Error');
    // 遍历所有在线可用的socket，调用其api告知
    for(const id in this._connDict){
      // 取出socket，无效则跳过
      const socket=this._connDict[id].socket;
      if(!socket) continue;

      // 调用其API，成败无所谓
      socket.callApi({
        api:'__broadMsg',
        heads:broadObj
      },params,(...all)=>{

      })
    }
  }
}
class c_service extends c_netPoint{
  constructor(conf){
    super();
    this.id          = conf.id;
    this._masterPort = conf.port;
    this._masterHost = conf.host||'127.0.0.1';
    this._certPath   = conf.certPath;
    this._listenPort = conf.listenPort; // 服务端口，默认随机
    this._pubAddr    = conf.pubAddr;    // 公网地址，可以为空

    // 广播处理函数集合，及内部广播接收方法
    this._broadFunc  = new c_apiFunc();
    this._broadFunc.reg(/^_{2}/,(heads,...params)=>{
      const func=this['_broad'+heads.broad];
      // 如果匹配到函数，执行
      if(typeof func==='function') func.apply(this,[heads,...params]);
    });
    // 开启tcp监听
    this._tcpListenBuild();

    // 启动及之后的每秒都尝试连接master
    this._connMaster();
    setInterval(()=>this._connMaster(),1000);
  }

  /**
   * 服务上线业务方法
   * @param id     服务id
   * @param verSt  版本戳
   * @param isBoot 是否是初次启动
   * @private
   */
  _serviceOn(id,verSt,isBoot){
    // master上线时
    if(id==='master'){
      // 遍历当前相关的服务
      for(const id in this._connDict){
        const econn=this._connDict[id];
        // 清除所有失败计次以及允许重连时间戳
        econn.offCount=0;
        econn.tryAgainSt=0;
        // 非强制更新订阅
        this.event.reBookService(id,false);
      }
    }
  }

  // 创建tcp链接，仅负责创建、连接，连接成功即可callback
  _socketTcpTry(conf,callback){
    let runCallback=!callback;
    const socket=net.connect(conf);
    socket.on('connect',()=>{
      if(!runCallback){
        runCallback=true;
        socket.removeAllListeners(socket.eventNames());
        callback(undefined,socket);
      }
    });
    socket.on('error',(err)=>{
      if(!runCallback){
        runCallback=true;
        socket.removeAllListeners(socket.eventNames());
        callback(err);
      }
    })
  }

  /**
   * 连接到master服务
   *
   * 在以下两种情况，会调用：
   * 1、tcp监听建立完毕；
   * 2、第一次getSocket调用；
   *
   * 情况2先于1发生时，获取不到tcpPort，会结束执行，等待情况1调用；
   * 情况1先于2发生时，建立字典缓存，故2不会再执行到此方法
   *
   */
  _connMaster(){
    // 无tcpPort时，不执行
    const tcpPort=this._tcpPort;
    if(!tcpPort) return;

    // 取出当前econn，没有则新建
    let econn=this._connDict.master;
    if(!econn) econn=this._connDict.master={};

    // 当前已有socket，不必继续
    if(econn.socket) return;

    const now=Date.now();

    // 当前不允许尝试连接
    if(econn.tryAgainSt && econn.tryAgainSt>now) return;

    // 建立过程中，需防止再次重连
    econn.tryAgainSt=Infinity;

    // 计算如果失败，下次允许的时间戳
    if(!econn.offCount) econn.offCount=0;
    const nextSt=econn.tryAgainSt=Date.now()+Math.min(econn.offCount++,60)*2000-300;

    // 建立TCP连接
    const port=this._masterPort;
    const host=this._masterHost;
    this._socketTcpTry({port,host},(err,socket)=>{
      // 连接失败，触发错误方法
      if(err){
        this.log('Conn Master Error: Build TCP Error');
        // 应用失败时间戳
        econn.tryAgainSt=nextSt;
        return;
      }

      // 丰富socket功能
      this._socketInit(socket);

      /* 整理注册信息，模板：regToMaster_req */
      const regObj=protoRoot.lookupType('ProtoBuf.Plant3.regToMaster_req').create();
      // id标识，计算机名，进程版本
      regObj.id       = this.id;
      regObj.hostname = etools.hostname;
      regObj.verSt    = this.verSt;
      // ip、端口记录、公网地址
      regObj.port     = tcpPort;
      regObj.ipRaw    = funcDict.ip2raw(socket.localAddress);
      regObj.pubAddr  = this._pubAddr;
      // 尝试读取本地证书并进行md5运算
      let localCert;
      if(this._certPath){
        try{
          localCert=fs.readFileSync(this._certPath);
          regObj.certMd5=etools.md5(localCert);
        } catch (e) {
          this.log('读取本地证书或计算MD5失败，按照没有证书处理');
        }
      }

      // 尝试向核心类注册
      socket.callApi('__connReg',regObj,(err,regBack)=>{
        // 连接到核心失败时，需断开本连接
        if (err) {
          this.log('Conn Master Error: reg to Master - '+err);
          // 应用失败时间戳
          econn.tryAgainSt=nextSt;
          return;
        }

        // 注册通过，放开重连
        econn.offCount=0;
        econn.tryAgainSt=0;

        // 如果服务器下发了新证书，且己方有存储路径，进行持久化
        const newCert=(regBack.cert&&regBack.cert.length>0)?regBack.cert:null;
        if(newCert && this._certPath){
          fs.writeFile(this._certPath,newCert,(err)=>{
            if(err) this.log('通讯证书写磁盘持久化失败，下次连接很可能会失败')
          });
        }

        // 写入id
        socket.id='master';

        // 如果要求加密，设定通讯加密方法
        if(regBack.crypto){
          // 优先使用新证书，其次使用本地读取的证书
          const certEnd=newCert||localCert;
          // 没有获取到，则报错
          if(!certEnd) throw new Error('服务器要求加密，但未下发新证书，本地证书读取也失败');
          this._socket_on(socket,regBack.verSt,regBack.skey,certEnd);
        }
        // 不要求加密
        else this._socket_on(socket,regBack.verSt);
      });

    })
  }

  /**
   * 主动建立与其他service连接
   *
   * @param mateBuild 对方信息
   * @param callback 回调函数
   */
  _tryToMate(mateBuild,callback){
    // 获取econn，没有则反馈错误
    const econn=this._connDict[mateBuild.mateId];
    if(!econn) {
      callback('no econn Info');
      return;
    }

    // 如果已经有socket，不再尝试
    if(econn.socket){
      // 检查是否有等待执行的方法
      const arry=econn.calkArry;
      delete econn.calkArry;
      if(arry) arry.forEach(f=>f(undefined,econn.socket));
      return;
    }

    // 3s内不可再次尝试
    const now=Date.now();
    if(econn.tryAgainSt&&econn.tryAgainSt>now){
      callback('tryAgain in short time');
      return;
    }

    // 更新允许再次连接的时间戳
    econn.tryAgainSt=now+3000;

    // 记录连接秘钥
    econn.connKey=mateBuild.connKey;

    // 成功获取socket之后的方法
    const funcGotSocket=(socket)=>{
      // 如果当前已有socket对象，先销毁
      if(econn.socket){
        // 销毁清空
        econn.socket.destroy();
        econn.socket=null;
      }

      // 丰富功能
      this._socketInit(socket);

      // 服务间注册
      socket.callApi('__mateReg',{
        _protoBufType:'ProtoBuf.Plant3.connMate_Build',
        verSt   : this.verSt,
        mateId  : this.id,
        connKey : econn.connKey
      },(err,mateBack)=>{
        // 注册失败
        if(err||!mateBack){
          callback('mate Reg Error');
        }
        // 注册成功
        else{
          // 写入id
          socket.id=mateBuild.mateId;
          callback(undefined,socket,mateBack.verSt);
        }
      })
    };

    // 先尝试连接对方内网ip
    this._socketTcpTry({
      port:mateBuild.port,
      host:funcDict.raw2ip(mateBuild.ipRaw)
    },(err,socket)=>{
      // 如果成功获取到socket，直接调用后续方法
      if(!err && socket) funcGotSocket(socket);
      // 失败，且对方没有公网接口，反馈失败
      else if(!mateBuild.pubAddr) callback('build Tcp Error: inner fail and no pubAddr');
      // 失败，但对方有公网接口，继续尝试
      else this._socketTcpTry({
          port:mateBuild.port,
          host:mateBuild.pubAddr
        },(err,socket)=>{
          // 依旧建立失败
          if(err||!socket)  callback('build Tcp Error: inner and pubAddr both Error');
          // 建立成功
          else funcGotSocket(socket);
        })
    })
  }

  // 服务获取econn实例
  _getSocket(id,callback){
    // 获取可信字典对象，没有则创建
    let econn=this._connDict[id];
    if(!econn) econn=this._connDict[id]={};

    // 如果有连接实例，直接反馈
    if(econn.socket) callback(undefined,econn.socket);
    // 没有连接实例，但有等待队列，加入
    else if(econn.calkArry) econn.calkArry.push(callback);
    // 如果此刻master不在线，报错
    else if(!this._connDict.master||!this._connDict.master.socket)
      callback('master off Line');
    // master可用，但要连接的刚刚尝试过，此刻不可再尝试，报错
    else if(econn.tryAgainSt&&econn.tryAgainSt>Date.now())
      callback('service Off Line');
    // 尝试连接到服务
    else {
      // 建立等待队列，此后无论如何，一定要调用socket_on/off来触发回调集合
      econn.calkArry=[callback];

      // 生成配对密码
      econn.connKey=crypto.randomBytes(32);

      // 连接建立失败的处理方法
      const func_fail=()=>{
        this.logDebug('主动尝试连接失败，但可能被动已链接: '+id);

        // 取出等待队列
        const arry=econn.calkArry;
        econn.calkArry=null;
        // 没有等待队列
        if(!arry){}
        // 有socket
        else if(econn.socket) arry.forEach(f=>f(undefined,econn.socket));
        // 没有socket
        else arry.forEach(f=>f('Build Tcp Error'));
      };

      // 调用master的API，协调建立连接
      this.logDebug('请求master协助连接service: '+id);
      this.callApi('master.__connMate',{
        _protoBufType:'ProtoBuf.Plant3.connMate_Build',
        mateId:id,
        connKey:econn.connKey
      },(err,mateBuild)=>{
        // 有错误信息，且发起尝试时间戳未被修改
        if(err) {
          this.logDebug('请求master协助连接service失败: '+id+' err: '+err);
          func_fail();
        }
        // 有mateBuild，表明需要己方尝试连接对方
        else if(!err && mateBuild) {
          // 尝试连接对方
          this.logDebug('自发主动尝试，连接service: '+mateBuild);
          this._tryToMate(mateBuild,(err)=>{
            if(err) func_fail();
          });
        }
      });
    }
  }

  // master下发广播消息
  _api__broadMsg(info,params,callback){
    // 只允许master调用
    if(info.socket.id!=='master'){
      callback('this API only call by Master');
      return
    }
    // 头信息必须有broad项目
    const broad=info.heads.broad;
    if(!broad) {
      callback('broadMsg API Heads need broad');
      return
    }

    // 获取广播方法
    const func=this._broadFunc.getFunc(broad);
    // 执行
    func(info.heads,...params);
  }

  // master下发要求连接到一般服务
  _api__tryToMate(info,[mateBuild],callback){
    // 只允许master调用
    if(info.socket.id!=='master'){
      callback('this API only call by Master');
      return
    }

    // 若字典项为空，则先创建
    if(!this._connDict[mateBuild.mateId]) this._connDict[mateBuild.mateId]={};

    // 调用内部方法
    this.logDebug('应邀主动尝试，连接service: '+mateBuild);
    this._tryToMate(mateBuild,(err,socket,verSt)=>{
      // 失败
      if(err||!socket||!verSt) callback(err||'build unknow Error');
      // 成功，调用认证上线方法
      else callback();
    })
  }

  // mate前来注册
  _api__mateReg(info,[mateInfo],callback){
    this.logDebug(`服务 ${mateInfo.mateId} 前来注册`);
    // 获取该mate信息，获取失败则回调错误
    const econn=this._connDict[mateInfo.mateId];
    if(!econn) {
      callback('mateInfo Error');
      return
    }

    /* 比对connKey */
    let keyOk=true;
    // 本地没有key或长度不一致，直接判定false
    if(!econn.connKey||econn.connKey.length!==mateInfo.connKey.length) keyOk=false;
    else{
      // 遍历每一位比较，遇到不同则判定false
      for(let i=0;i<mateInfo.connKey.length;i++){
        if(mateInfo.connKey[i]!==econn.connKey[i]) {
          keyOk=false;
          break;
        }
      }
    }

    // 若不匹配，反馈失败
    if(!keyOk) callback('connKey Error');
    // 匹配成功
    else {
      // 反馈成功
      callback(undefined,{
        _protoBufType:'ProtoBuf.Plant3.connMate_Build',
        verSt   : this.verSt
      });
      // 写入id
      info.socket.id=mateInfo.mateId;
      // 调用上线方法
      this._socket_on(info.socket,mateInfo.verSt);
    }
  }

  // 收到服务可用广播，即master认为该服务重新上线
  _broad__serviceUsable(heads,serviceId,verSt){
    // 未曾有过相关信息，什么也不做
    const econn=this._connDict[serviceId];
    if(!econn) return;

    // 如果通知版本与当前在线版本相同，什么也不错
    if(verSt===econn.verSt && econn.socket) return;

    /* 得知该服务版本发生了变更，或重新上线 */
    this.logDebug(`服务${serviceId}可用，版本${verSt}`);

    // 更新该service的订阅
    this.event.reBookService(serviceId,true);
  }
}

/**
 * 将proto对象转json方法
 */
function protoObj2jsonObj(obj){
  switch (etools.protoType(obj)) {
    case '[object Null]':
    case '[object RegExp]':
    case '[object Uint8Array]':
    case '[object Undefined]':
    case '[object Boolean]':
    case '[object Number]':
    case '[object String]': return obj;
    case '[object Array]': return obj.map(i=>{return protoObj2jsonObj(i)});
    case '[object Object]':
      // 本就是jsonObj，原样返回
      if(obj.constructor.name==='Object') return obj;

      // Long处理
      if(obj.constructor.name==='Long') return obj.toNumber();

      // 构造新jsonObj
      const jsonObj={
        _protoBufType:obj.constructor.name
      };
      // 遍历赋值
      for(const key in obj){
        if(!obj.hasOwnProperty(key)) continue;
        jsonObj[key]=protoObj2jsonObj(obj[key]);
      }

      // 反馈
      return jsonObj;
  }
}

// 对外输出
module.exports={
  isDebug,
  c_fluentCache,
  c_master,
  c_service,
  c_apiConn,
  c_econn,
  c_apiFunc,
  c_event,
  addProtoFile(filepath){
    protoRoot.loadSync(filepath);
  },  // 添加proto文件
  protoObj2jsonObj
};
